@@ -69,6 +69,8 @@ int shell_opt;
 int termios_set;
 struct termios canonical_tty;
 
+static char *shell, *shell_base;
+
 /* forwards */
 
 static void usage();
@@ -135,8 +137,12 @@ main(int argc, char *argv[]) {
 		err(1, "getrlimit");
 	/* guard against unrealistic replies */
 	open_max = min(65536, (unsigned)rl.rlim_cur);
-	if (open_max == 0)
-		open_max = 65536;
+	/* maxfiles is ulimited on modern MacOS */
+	if (open_max < 65536) {
+		rl.rlim_cur = (rlim_t)65536;
+		if (setrlimit(RLIMIT_NOFILE, &rl) == 0 || open_max == 0)
+			open_max = 65536;
+	}
 #else /* BSD */
 	if (getrlimit(RLIMIT_NOFILE, &rl) == -1)
 		err(1, "getrlimit");
@@ -155,15 +161,18 @@ main(int argc, char *argv[]) {
 	/* ensure a shell is available to use */
 	setenv("SHELL", "/bin/sh", 0);
 
+	shell = getenv("SHELL");
+	shell_base = strdup(shell);
+	if (shell_base == NULL)
+		err(1, "cannot duplicate string");
+	shell_base = basename(shell_base);
+
 	/* sequential scan may depend on a 0 at the end */
 	files = calloc(open_max+1, sizeof(WatchFile *));
 
 	if ((kq = kqueue()) == -1)
 		err(1, "cannot create kqueue");
 
-	if (fcntl(kq, F_SETFD, FD_CLOEXEC) == -1)
-		warn("failed to set FD_CLOEXEC to kqueue descriptor");
-
 	/* expect file list from a pipe */
 	if (isatty(fileno(stdin)))
 		usage();
@@ -236,7 +245,7 @@ handle_exit(int sig) {
 	terminate_utility();
 
 	if ((sig == SIGINT || sig == SIGHUP))
-	    exit(0);
+	    _exit(0);
 	else
 	    raise(sig);
 }
@@ -244,6 +253,7 @@ handle_exit(int sig) {
 void
 proc_exit(int sig) {
 	int status;
+	int saved_errno = errno;
 
 	if ((!noninteractive_opt) && (termios_set))
 		tcsetattr(STDIN_FILENO, TCSADRAIN, &canonical_tty);
@@ -256,20 +266,26 @@ proc_exit(int sig) {
 			print_child_status(child_status);
 
 		if WIFSIGNALED(child_status)
-			exit(128 + WTERMSIG(child_status));
+			_exit(128 + WTERMSIG(child_status));
 		else
-			exit(WEXITSTATUS(child_status));
+			_exit(WEXITSTATUS(child_status));
 	}
+	/* restore errno so that the resuming code is unimpacted. */
+	errno = saved_errno;
 }
 
 void
 print_child_status(int status) {
+	int len;
+	char buf[2048];
+
 	if WIFSIGNALED(status)
-		fprintf(stdout, "%s terminated by signal %d\n",
-		    basename(getenv("SHELL")), WTERMSIG(status));
+		len = snprintf(buf, sizeof(buf), "%s terminated by signal %d\n",
+		    shell_base, WTERMSIG(status));
 	else
-		fprintf(stdout, "%s returned exit code %d\n",
-		    basename(getenv("SHELL")), WEXITSTATUS(status));
+		len = snprintf(buf, sizeof(buf), "%s returned exit code %d\n",
+		    shell_base, WEXITSTATUS(status));
+	write(STDOUT_FILENO, buf, len);
 }
 
 /*
@@ -414,7 +430,7 @@ run_utility(char *argv[]) {
 		arg_buf = malloc(ARG_MAX);
 		new_argv = calloc(argc+1, sizeof(char *));
 		realpath(leading_edge->fn, arg_buf);
-		new_argv[0] = getenv("SHELL");
+		new_argv[0] = shell;
 		new_argv[1] = "-c";
 		new_argv[2] = argv[0];
 		new_argv[3] = arg_buf;
